@@ -13,7 +13,7 @@
 
 ##Author: Christos Zamantzas <christos.zamantzas@gmail.com>
 
-Version = '2.4'
+Version = '2.5'
 
 import os
 import sys
@@ -67,10 +67,10 @@ def showQuickMessage(message):
                org.freedesktop.Notifications.SystemNoteInfoprint \
                string:"%s"' % message)          
 
-def checkConfigFile():
-    ''' Check for configulation files. '''               
+def checkConfigFileAvailability():
+    ''' Check for configuration files and select which one to be used. '''               
 
-    global Path, PathSystem
+    global Path, Config
     if checkFile(PathTheme,ConfigTheme) == True:
        Path = PathTheme
        Config = ConfigTheme
@@ -78,7 +78,7 @@ def checkConfigFile():
        Path = PathSystem
        Config = ConfigSystem
     else:
-       message = 'WARNING: Unable to find any configuration file. Exiting..'
+       message = 'ERROR: Unable to find any configuration file. Exiting..'
        showMessage(message)
        sys.exit(1)
 
@@ -99,10 +99,21 @@ def checkTactileInstalled():
        return True
     else: 
        return False
-        
-def getCurrent(): #FIXME: add new confiVars for version 20.2010.36-maemo16
+
+def copyConfigFile():
+    '''Copy the system/theme congiguration files for later editing'''
+    
+    os.system('cp %s %s' % (PathSystem + ConfigSystem, PathTemp + ConfigTempSystem))
+    if checkFile(PathTheme,ConfigTheme) == True: 
+       os.system('cp %s %s' % (PathTheme + ConfigTheme, PathTemp + ConfigTempTheme))
+       message = 'INFO: The application found a settings file in the theme folder.\nThe application will update both system and theme files with any changes requested.'
+       showMessage(message)
+    else:
+       pass
+       
+def getCurrent():
     '''Read the values currently in the config file'''
-     
+
     if checkFile(PathTheme,ConfigTheme) == True:
        Path = PathTheme
        Config = ConfigTheme
@@ -112,12 +123,12 @@ def getCurrent(): #FIXME: add new confiVars for version 20.2010.36-maemo16
        Path = PathSystem
        Config = ConfigSystem
 
-    #define variables to readout
+    ##define variables to readout
     global dataVars
     dataVars={}
     configVars = ['zoom_on_press', 'parallax', 'blurless', 'blurless_saturation', 'taskswitcher', 'zaxisrotation', 'forcerotation', 'tactilepopups']
 
-    #readout the variables               
+    ##readout the variables               
     try:
        f = open(Path + Config, 'r')
        string=f.read().replace(' ','')
@@ -127,14 +138,37 @@ def getCurrent(): #FIXME: add new confiVars for version 20.2010.36-maemo16
           if line[0] in configVars:
              dataVars[line[0]]=line[1]
     except IOError:
-       message = 'Reading the configuration settings gave a FATAL ERROR while loading\nConfiguration file does not exist or cannot be read.'
+       message = 'ERROR: Reading the configuration settings gave a FATAL ERROR while loading\nConfiguration file does not exist or cannot be read.'
        showMessage(message)
        sys.exit(1)
     except:
-       message = 'Reading the configuration settings gave a FATAL ERROR while loading\nSettings are corrupted.'
+       message = 'ERROR: Reading the configuration settings gave a FATAL ERROR while loading\nSettings are corrupted.'
        showMessage(message)
        sys.exit(1)
-                   
+       
+    ##Check if variables exist in the file
+    global fixNeeded
+    fixNeeded = 'false'
+
+    if 'zoom_on_press' not in dataVars: 
+       fixNeeded = 'true'
+    elif 'parallax' not in dataVars: 
+       fixNeeded = 'true'
+    elif 'blurless' not in dataVars: 
+       fixNeeded = 'true'
+    elif 'blurless_saturation' not in dataVars: 
+       fixNeeded = 'true'
+    elif 'taskswitcher' not in dataVars:
+       fixNeeded = 'true'
+    elif 'zaxisrotation' not in dataVars: 
+       fixNeeded = 'true'
+    elif 'forcerotation' not in dataVars: 
+       fixNeeded = 'true'
+    elif 'tactilepopups' not in dataVars: 
+       fixNeeded = 'true'
+    else: 
+       fixNeeded = 'false'
+   
 def doUpdateConfigSystem():            
     ''' Update the config file of the system with the edited. '''
 
@@ -156,7 +190,146 @@ def doUpdateConfigTheme():
        return False
     else: 
        return True
+
+class CSSUfeaturesFixConfig(QtGui.QMainWindow):
+
+    def __init__(self, parent=None):
+       QtGui.QMainWindow.__init__(self,parent)
+       self.parent = parent
+       self.setAttribute(Qt.WA_Maemo5AutoOrientation, True)
+       self.setAttribute(Qt.WA_Maemo5StackedWindow, True)
+       self.setWindowTitle("Missing Configuration")
+       self.showFullScreen()
+
+       aboutScrollArea = QScrollArea(self)
+       aboutScrollArea.setWidgetResizable(True)
+       awidget = QWidget(aboutScrollArea)
+       awidget.setSizePolicy( QSizePolicy.Expanding, QSizePolicy.Expanding )
+       aboutScrollArea.setSizePolicy( QSizePolicy.Expanding, QSizePolicy.Expanding )
+       try:
+          scroller = aboutScrollArea.property("kineticScroller").toPyObject()
+          scroller.setEnabled(True)
+       except:
+          pass
+
+       aboutLayout = QVBoxLayout(awidget)
+
+       aboutIcon = QLabel()
+       aboutIcon.setPixmap( QIcon.fromTheme('CSSUfeatures').pixmap(64,64) )
+       aboutIcon.setAlignment( Qt.AlignCenter or Qt.AlignHCenter )
+       #aboutIcon.resize(128,128)
+       aboutLayout.addWidget(aboutIcon)
        
+       aboutLabel = QLabel(''' <center><b>Disclaimer/Warning</b>
+                               <br>The transitions.ini file in this system
+                               <br>does not have all the needed options.
+                               <br>A repair operation can be initiated but
+                               <br>the result cannot be guaranteed.
+                               <br>
+                               <br>Meaning, the system might enter in a
+                               <br>reboot loop and an advanced repair or
+                               <br>reflash might be needed.
+                               </center>''')
+       aboutLayout.addWidget(aboutLabel)
+       
+       self.fix_button = QPushButton('Fix Config')
+       self.fix_button.clicked.connect(self.fixConfigVariables)
+       self.cancel_button = QPushButton('Exit')
+       self.cancel_button.clicked.connect(self.cancel)
+       awidget2 = QWidget()
+       buttonLayout = QHBoxLayout(awidget2)        
+       buttonLayout.addWidget(self.fix_button)
+       buttonLayout.addWidget(self.cancel_button)
+       aboutLayout.addWidget(awidget2)
+        
+       awidget.setLayout(aboutLayout)
+       aboutScrollArea.setWidget(awidget)
+       self.setCentralWidget(aboutScrollArea)
+       
+       self.show()        
+    
+    def cancel(self):
+       
+       sys.exit(1)
+
+    def fixConfigVariables(self): ##FIXME: make a loop on variables than this spaghetti code!
+       
+       if Path == PathSystem:
+          configLocation = 'configSystem'
+       elif Path == PathTheme:
+          configLocation = 'configTheme'
+       else: 
+          configLocation = ''
+       print "fixing config file: ", configLocation
+          
+       message = 'Start of fixing\n'
+       if 'zoom_on_press' not in dataVars:
+          os.system('sudo %s%s addzoom_on_press %s' % (PathApp, updateSettings, configLocation))
+          message += 'WARNING: Added the zoom_on_press option..\n'
+          print message
+       else:
+          print 'zoom_on_press exists!!'
+
+       if 'parallax' not in dataVars:
+          os.system('sudo %s%s addparallax %s' % (PathApp, updateSettings, configLocation))
+          message += 'WARNING: Added the parallax option..\n'
+          print message
+       else:
+          print 'parallax exists!!'
+
+       if 'blurless' not in dataVars:
+          os.system('sudo %s%s addblurless %s' % (PathApp, updateSettings, configLocation))
+          message += 'WARNING: Added the blurless option..\n'
+          print message
+       else:
+          print 'blurless exists!!'
+                      
+       if 'blurless_saturation' not in dataVars:
+          os.system('sudo %s%s addblurless_saturation %s' % (PathApp, updateSettings, configLocation))
+          message += 'WARNING: Added the blurless_saturation option..\n'
+          print message
+       else:
+          print 'blurless_saturation exists!!'
+                      
+       if 'taskswitcher' not in dataVars:
+          os.system('sudo %s%s addtaskswitcher %s' % (PathApp, updateSettings, configLocation))
+          message += 'WARNING: Added the taskswitcher option..\n'
+          print message
+       else:
+          print 'taskswitcher exists!!'
+                      
+       if 'zaxisrotation' not in dataVars:
+          os.system('sudo %s%s addzaxisrotation %s' % (PathApp, updateSettings, configLocation))
+          message += 'WARNING: Added the zaxisrotation option..\n'
+          print message
+       else:
+          print 'zaxisrotation exists!!'
+                      
+       if 'forcerotation' not in dataVars:
+          os.system('sudo %s%s addforcerotation %s' % (PathApp, updateSettings, configLocation))
+          message += 'WARNING: Added the forcerotation option..\n'
+          print message
+       else:
+          print 'forcerotation exists!!'
+                      
+       if 'tactilepopups' not in dataVars:
+          os.system('sudo %s%s addtactilepopups %s' % (PathApp, updateSettings, configLocation))
+          message += 'WARNING: Added the tactilepopups option..\n'
+          print message
+       else:
+          print 'tactilepopups exists!!'
+
+       message += 'End of fixing\n\n'
+       message += 'The application will now exit..'
+       #aboutLabel = QLabel(''' <center><b>UPDATING</b><br>%s</center>''' % message)  
+       showMessage(message)
+       getCurrent()
+       copyConfigFile()
+       time.sleep(5)
+       # self.close()   
+       sys.exit(1)
+
+
 class CSSUfeaturesAbout(QtGui.QMainWindow):
     '''About Window'''
     def __init__(self, parent=None):
@@ -182,7 +355,7 @@ class CSSUfeaturesAbout(QtGui.QMainWindow):
        aboutLayout = QVBoxLayout(awidget)
 
        aboutIcon = QLabel()
-       aboutIcon.setPixmap( QIcon.fromTheme('CSSUfeatures').pixmap(48,48) )
+       aboutIcon.setPixmap( QIcon.fromTheme('CSSUfeatures').pixmap(64,64) )
        aboutIcon.setAlignment( Qt.AlignCenter or Qt.AlignHCenter )
        aboutIcon.resize(128,128)
        aboutLayout.addWidget(aboutIcon)
@@ -254,30 +427,37 @@ class CSSUfeaturesMainWindow(QtGui.QMainWindow):
        
          ##Connect Menu Buttons 
        QtCore.QObject.connect(self.ui.actionQuit, QtCore.SIGNAL('triggered()'), QtGui.qApp, QtCore.SLOT('quit()'))
-       QtCore.QObject.connect(self.ui.actionAbout, QtCore.SIGNAL('triggered()'), self.doAbout)
-       QtCore.QObject.connect(self.ui.actionRestoreDefaults, QtCore.SIGNAL('triggered()'), self.doRestoreDefaults)
-       QtCore.QObject.connect(self.ui.actionRebootDevice, QtCore.SIGNAL('triggered()'), self.doRebootDevice)
-       QtCore.QObject.connect(self.ui.actionAddPowerKeyEntry, QtCore.SIGNAL('triggered()'), self.doAddPowerKeyEntry)
-       QtCore.QObject.connect(self.ui.actionRemovePowerKeyEntry, QtCore.SIGNAL('triggered()'), self.doRemovePowerKeyEntry)
+       QtCore.QObject.connect(self.ui.actionAbout, QtCore.SIGNAL('triggered()'), self.showAbout)
+       QtCore.QObject.connect(self.ui.actionRestoreDefaults, QtCore.SIGNAL('triggered()'), self.restoreDefaultConfigFile)
+       QtCore.QObject.connect(self.ui.actionRebootDevice, QtCore.SIGNAL('triggered()'), self.rebootDevice)
+       QtCore.QObject.connect(self.ui.actionAddPowerKeyEntry, QtCore.SIGNAL('triggered()'), self.addPowerKeyEntry)
+       QtCore.QObject.connect(self.ui.actionRemovePowerKeyEntry, QtCore.SIGNAL('triggered()'), self.removePowerKeyEntry)
          ##Enable only the valid menu entry 
        if checkFile(PathSystemUi, ConfigSystemUi) == True:
           self.ui.actionAddPowerKeyEntry.setEnabled(False)
        else:
           self.ui.actionRemovePowerKeyEntry.setEnabled(False)
        
-       ## Check which config file to use.
-       checkConfigFile()
-       ##Set current values in the GUI.
-       self.doSetCurrent()
+       ##Check which config file to use.
+       checkConfigFileAvailability()
+       ##Get the values from the configuration file.
+       getCurrent()
        
+       ##Check if all values exist
+       if fixNeeded == 'true':   
+          print "fix is needed"
+          self.fixConfigFileVariables()
+       else:
+          self.doSetCurrent()
+          
     ##Create Methods
-             
-    def doAbout(self):
+
+    def showAbout(self):
   
        stackwindow = CSSUfeaturesAbout(self)
        stackwindow.show()
 
-    def doAddPowerKeyEntry(self):
+    def addPowerKeyEntry(self):
        '''Add entry in the Power Key menu'''
        
        os.system('sudo %s%s addPowerKeyEntry' % (PathApp, updateSettings) )
@@ -287,7 +467,7 @@ class CSSUfeaturesMainWindow(QtGui.QMainWindow):
        message = 'Added entry in the Power Key Menu'
        showQuickMessage(message)
 
-    def doRemovePowerKeyEntry(self):
+    def removePowerKeyEntry(self):
        '''Remove entry from the Power Key menu'''
 
        os.system('sudo %s%s deletePowerKeyEntry' % (PathApp, updateSettings) )
@@ -296,8 +476,8 @@ class CSSUfeaturesMainWindow(QtGui.QMainWindow):
        os.system('sudo %s%s updatePowerKeyEntry' % (PathApp, updateSettings) )
        message = 'Removed entry from the Power Key Menu'
        showQuickMessage(message)
-                  
-    def doRestoreDefaults(self):
+
+    def restoreDefaultConfigFile(self):
        ''' Restore the default configuration file.'''
        
        message = 'Please wait. Update in progress..'
@@ -306,11 +486,12 @@ class CSSUfeaturesMainWindow(QtGui.QMainWindow):
        try:
           os.system('sudo %s%s copyDefault' % (PathApp, updateSettings))
        except:
-          message = 'The default transitions.ini file could not be restored to the system.'
+          message = 'WARNING: The default transitions.ini file could not be restored to the system.'
        else: 
           message = 'The default transitions.ini file have been restored to the system.\n\nINFO: Some of its settings will appear on the next reboot.'
-       
-    def doRebootDevice(self):
+       showMessage(message)
+
+    def rebootDevice(self):
        '''Reboot Device for the settings to take effect.'''
        
        message = 'INFO: Device is going to be restarted.\nThe application is exiting..'
@@ -321,17 +502,17 @@ class CSSUfeaturesMainWindow(QtGui.QMainWindow):
 
     def doSetCurrent(self):
        ''' Set CURRENT values in the GUI''' 
-
-       #Get the values from the configuration file.
-       getCurrent()
        
        ##Check if the tactile package is installed
        if checkTactileInstalled() == False:
           self.ui.comboBoxTactile.setEnabled(False)
        else:
           pass
-
-       #Set them in the GUI.
+          
+       ##Get the values from the configuration file.
+       getCurrent()
+          
+       ##Set them in the GUI.
        try:
           self.ui.comboBoxZoomPress.setCurrentIndex(int(dataVars['zoom_on_press']))
           self.ui.comboBoxBlurless.setCurrentIndex(int(dataVars['blurless']))
@@ -345,7 +526,7 @@ class CSSUfeaturesMainWindow(QtGui.QMainWindow):
           message = 'The CURRENT configuration options are shown.'
           showQuickMessage(message)
        except:
-          message = 'Reading the configuration settings gave a FATAL ERROR while loading. Not all settings are available.\n\nExiting..'
+          message = 'ERROR: Reading the configuration settings gave a FATAL ERROR while loading the transitions.ini file. Not all settings are available.\n\nExiting..'
           showMessage(message)
           sys.exit(1)
            
@@ -357,7 +538,7 @@ class CSSUfeaturesMainWindow(QtGui.QMainWindow):
           RowsStatusMenuPortrait = os.popen('gconftool-2 -g /apps/osso/hildon-status-menu/view/number_of_rows_portrait').read()
           DesktopScroll = os.popen('gconftool-2 -g /apps/osso/hildon-desktop/scroll_vertical').read()
            
-          #print FMTX, AppMenuEditor, DesktopEditor, DesktopScroll, RowsStatusMenu, RowsStatusMenuPortrait
+          # print FMTX, AppMenuEditor, DesktopEditor, DesktopScroll, RowsStatusMenu, RowsStatusMenuPortrait
            
           if FMTX == 'false\n':
              self.ui.comboBoxFMTX.setCurrentIndex(0)
@@ -379,10 +560,10 @@ class CSSUfeaturesMainWindow(QtGui.QMainWindow):
           self.ui.doubleSpinBoxRowsStatusMenu.setValue(int(RowsStatusMenu))
           self.ui.doubleSpinBoxRowsStatusMenuPortrait.setValue(int(RowsStatusMenuPortrait))
        except:
-          message = 'Reading the configuration settings gave a FATAL ERROR while reading the gconf values. Not all settings are available.\n\nExiting..'
+          message = 'ERROR: Reading the configuration settings gave a FATAL ERROR while reading the gconf values. Not all settings are available.\n\nExiting..'
           showMessage(message)
           sys.exit(1)
-
+       
     def doSetDefault(self):  
        '''Set DEFAULT values in the GUI''' 
 
@@ -427,15 +608,15 @@ class CSSUfeaturesMainWindow(QtGui.QMainWindow):
        if self.ui.comboBoxAppMenuEditor.currentIndex() == 0:
           AppMenuEditor = 'false'
        else:    
-          AppMenuEditor =  'true'
+          AppMenuEditor = 'true'
        if self.ui.comboBoxDesktopEditor.currentIndex() == 0:
           DesktopEditor = 'false'
        else:
-          DesktopEditor =  'true'
+          DesktopEditor = 'true'
        if self.ui.comboBoxDesktopScroll.currentIndex() == 0:
           DesktopScroll = 'false'
        else:
-          DesktopScroll =  'true'
+          DesktopScroll = 'true'
           
        RowsStatusMenu = self.ui.doubleSpinBoxRowsStatusMenu.value()
        RowsStatusMenuPortrait = self.ui.doubleSpinBoxRowsStatusMenuPortrait.value()
@@ -446,9 +627,9 @@ class CSSUfeaturesMainWindow(QtGui.QMainWindow):
        os.system('gconftool-2 -s /apps/osso/hildon-desktop/key-actions/disable_edit -t bool %s' % DesktopEditor)
        os.system('gconftool-2 -s /apps/osso/hildon-status-menu/view/number_of_rows -t int %s' % RowsStatusMenu)
        os.system('gconftool-2 -s /apps/osso/hildon-status-menu/view/number_of_rows_portrait -t int %s' % RowsStatusMenuPortrait)
-       os.system('gconftool-2 --set /apps/osso/hildon-desktop/scroll_vertical -t bool %s' % DesktopScroll)
+       os.system('gconftool-2 -s /apps/osso/hildon-desktop/scroll_vertical -t bool %s' % DesktopScroll)
        
-       #Update of the System config file.
+       ##Update of the System config file.
        os.rename( PathTemp + ConfigTempSystem, PathTemp + ConfigTempSystem + "~" )
        destination = open( PathTemp + ConfigTempSystem, "w" )
        source = open( PathTemp + ConfigTempSystem + "~", "r" )
@@ -457,9 +638,9 @@ class CSSUfeaturesMainWindow(QtGui.QMainWindow):
              destination.write('zoom_on_press = %s\n' % ZoomPress)
           elif Line.startswith("parallax"):
              destination.write('parallax = %s\n' % Parallax)
-          elif Line.startswith("blurless "):
+          elif Line.startswith("blurless ") or Line.startswith("blurless="):
              destination.write('blurless = %s\n' % Blurless)
-          elif Line.startswith("blurless_saturation "):
+          elif Line.startswith("blurless_saturation"):
              destination.write('blurless_saturation = %s\n' % BlurlessSaturation)
           elif Line.startswith("taskswitcher"):
              destination.write('taskswitcher = %s\n' % TaskSwitcher)
@@ -477,7 +658,7 @@ class CSSUfeaturesMainWindow(QtGui.QMainWindow):
        doUpdateConfigSystem()
        message = 'The new settings have been successfully saved.'
        
-       #Update of the Theme config file (if it exists).
+       ##Update of the Theme config file (if it exists).
        if checkFile(PathTheme,ConfigTheme) == True:
 
           os.rename( PathTemp + ConfigTempTheme, PathTemp + ConfigTempTheme + "~" )
@@ -488,9 +669,9 @@ class CSSUfeaturesMainWindow(QtGui.QMainWindow):
                 destination.write('zoom_on_press = %s\n' % ZoomPress)
              elif Line.startswith("parallax"):
                 destination.write('parallax = %s\n' % Parallax)
-             elif Line.startswith("blurless "):
+             elif Line.startswith("blurless ") or Line.startswith("blurless="):
                 destination.write('blurless = %s\n' % Blurless)
-             elif Line.startswith("blurless_saturation "):
+             elif Line.startswith("blurless_saturation"):
                 destination.write('blurless_saturation = %s\n' % BlurlessSaturation)
              elif Line.startswith("taskswitcher"):
                 destination.write('taskswitcher = %s\n' % TaskSwitcher)
@@ -510,6 +691,12 @@ class CSSUfeaturesMainWindow(QtGui.QMainWindow):
 
        showQuickMessage(message)
 
+    def fixConfigFileVariables(self): 
+       '''show window with fix options'''
+       
+       stackwindow = CSSUfeaturesFixConfig(self)
+       stackwindow.show()
+
 class CSSUfeaturesMain(): 
      
     ##Check with what priviledges the GUI has been executed
@@ -522,7 +709,7 @@ class CSSUfeaturesMain():
     
     ##Check if CSSU is installed
     if checkCSSUinstalled() == False:
-       message = 'ERROR: The Community SSU is not installed.\nSome features might not work correctly..'
+       message = 'WARNING: The Community SSU is not installed.\nSome features might not work correctly..'
        showMessage(message)
        #sys.exit(1)
     else:
@@ -535,15 +722,9 @@ class CSSUfeaturesMain():
        pass
      
     ##Copy the system/theme congiguration files for later editing
-    os.system('cp %s %s' % (PathSystem + ConfigSystem, PathTemp + ConfigTempSystem))
-    if checkFile(PathTheme,ConfigTheme) == True: 
-       os.system('cp %s %s' % (PathTheme + ConfigTheme, PathTemp + ConfigTempTheme))
-       message = 'INFO: The application found a settings file in the theme folder.\nThe application will update both system and theme files with any changes requested.'
-       showMessage(message)
-    else:
-       pass
-     
-    ##Open Dialog Window
+    copyConfigFile()
+    
+    ##Open Main Window
     app = QtGui.QApplication(sys.argv)
     myapp = CSSUfeaturesMainWindow()
     myapp.show()
